using System.Xml;		// for XMLTextWriter and XMLTextReader
using System.IO;		// for StreamReader and StreamWriter

// SAS Enterprise Guide addins namespace
using SAS.Shared.AddIns;

namespace EGAddin.CalculateLag
{
	/// <summary>
	/// Calculate Lag/Dif custom task for use in SAS Enterprise Guide or SAS Add-In for Microsoft Office
	/// </summary>
	public class Lag : SAS.Shared.AddIns.ISASTaskAddIn, SAS.Shared.AddIns.ISASTaskDescription, SAS.Shared.AddIns.ISASTask
	{
		// change these values for your task
		#region private members for boilerplate values
		
		// replace these boilerplate values with values that apply to your task
		private string sLabel = "Calculate Lag and Dif";
		private string sAddInName = "Compute Lag and Dif for a column";
		private string sAddInDescription = "Compute Lag and Dif for a column";
		private string sProductsReq = "BASE";
		private string sProductsOpt = "";
		private string sTaskName= "Compute Lag and Dif";
		private string sCategory = "Add-in Examples";
		private string sTaskDescription = "Compute Lag and Dif for a column";
		private string sFriendlyName = "Compute Lag and Dif";
		private string sWhatIsDescription = "Compute Lag and Dif for a column";
		
		#endregion

		#region Properties and members
		// store reference to the application Consumer object
		private ISASTaskConsumer consumer = null;

		public enum eFunction
		{
			Lag,
			Dif
		}

		private eFunction _function=eFunction.Lag;
		/// <summary>
		/// Which function to calc: lag or dif
		/// </summary>
		public eFunction Function
		{
			get { return _function; }
			set { _function = value; }
		}

		private int _n = 1;
		/// <summary>
		/// By how many records to lag/dif
		/// </summary>
		public int n 
		{
			get { return _n; }
			set { value = _n; }
		}

		private string _lagcolumn="";
		/// <summary>
		/// Column to Lag/dif
		/// </summary>
		public string LagColumn
		{
			get { return _lagcolumn; }
			set { _lagcolumn = value; }
		}
		#endregion

		#region Ctor
		public Lag()
		{
		}
		#endregion

		#region ISASTaskAddIn Members

		public bool VisibleInManager
		{
			get
			{				
				return true;
			}
		}

		public string AddInName
		{
			get
			{				
				return sAddInName;
			}
		}

		public void Disconnect()
		{
			// perform cleanup tasks
			consumer = null;
		}

		public int Languages(out string[] Items)
		{		
			// by default, we support English 
			// Add more languages as needed
			Items = new string[] {"en-US"};
			return 1;
		}

		public string AddInDescription
		{
			get
			{				
				return sAddInDescription;
			}
		}

		public bool Connect(ISASTaskConsumer Consumer)
		{
			// perform any initialization needed when the application connects
			consumer = Consumer;
	
			// this is a good time to get the consumer.ActiveData, if your task requires it.

			return true;
		}

		public string Language
		{
			set
			{
				// if you support multiple languages, add handler here
			}
		}

		#endregion

		#region ISASTaskDescription Members

		public string ProductsRequired
		{
			get
			{
				// What SAS products are required for this task to run?
				return sProductsReq;
			}
		}

		public bool StandardCategory
		{
			get
			{
				// used typically only by SAS-supplied tasks			
				return false;
			}
		}

		public bool GeneratesListOutput
		{
			get
			{
				// Does this task generate ODS-style output?
				return true;
			}
		}

		public string TaskName
		{
			get
			{
				return sTaskName;
			}
		}

		public string Validation
		{
			get
			{
				// Add a validation string that makes sense for your organization
				// For example, "dev", "test", or "prod"
				return "";
			}
		}

		public string TaskCategory
		{
			get
			{
				// PLACEHOLDER: replace with your own category
				return sCategory;
			}
		}

		public string IconAssembly
		{
			get
			{
				// return the full path/name of this assembly, assuming that the
				// icon is embedded within the assembly
				return System.Reflection.Assembly.GetExecutingAssembly().Location;
			}
		}

		public string Clsid
		{
			get
			{
				// ClassID GUID generated by the template
				return "9F259B91-8490-4457-B9A9-CF088713AE28";
			}
		}

		public SAS.Shared.AddIns.ShowType TaskType
		{
			get
			{				
				return SAS.Shared.AddIns.ShowType.Wizard;
			}
		}

		public bool RequiresData
		{
			get
			{
				// Does your task require input data from the application?
				return true;
			}
		}

		public string IconName
		{
			get
			{
				// return the name of the icon within this assembly
				// including namespace qualifiers
				return "EGAddin.CalculateLag.function.ico";
			}
		}

		public int MinorVersion
		{
			get
			{
				return 0;
			}
		}

		public int MajorVersion
		{
			get
			{
				return 1;
			}
		}

		public int NumericColsRequired
		{
			get
			{
				// How many numeric variables are required in input data, if any?
				return 0;
			}
		}

		public string TaskDescription
		{
			get
			{				
				return sTaskDescription;
			}
		}

		public bool GeneratesSasCode
		{
			get
			{
				// Return true if your task generates SAS program code
				return true;
			}
		}

		public string FriendlyName
		{
			get
			{
				// Replace with a user-friendly name for your task
				return sFriendlyName;
			}
		}

		public string ProcsUsed
		{
			get
			{
				// What SAS procedures are used in this task?
				return "";
			}
		}

		public string WhatIsDescription
		{
			get
			{
				// Longer description for your task
				return sWhatIsDescription;
			}
		}

		public string ProductsOptional
		{
			get
			{
				// What SAS products are optionally used by this task?
				return sProductsOpt;
			}
		}

		#endregion

		#region ISASTask Members

		public string RunLog
		{
			get
			{
				// if your task does not generate SAS code, you can supply your own
				// log text to record the work completed.
				return "";
			}
		}

		// manage the state of the task when serializing to and from the project
		// makes use of some helper methods: WriteXML and ReadXML
		public string XmlState
		{
			get
			{
				return WriteXML();
			}
			set
			{
				ReadXML(value);
			}
		}

		public void Terminate()
		{
			// Cleanup as needed
		}

		private string MakeOutDataName()
		{
			return string.Format("WORK.{0}_{1}",this.consumer.ActiveData.Member,_function==eFunction.Lag ? "LAG" : "DIF");
		}

		public SAS.Shared.AddIns.OutputData OutputDataInfo(int Index, out string Source, out string Label)
		{
			Source = MakeOutDataName();
			Label = Source;
			return SAS.Shared.AddIns.OutputData.Libref;
		}

		public string SasCode
		{
			get
			{				
				string outdata=MakeOutDataName();
				string codetemplate = Global.ReadFileFromAssembly("EGAddin.CalculateLag.CalculateLag.sas");
				codetemplate = codetemplate.Replace("{outdata}",outdata);
				codetemplate = codetemplate.Replace("{inputdata}",string.Format("{0}.{1}",this.consumer.ActiveData.Library,this.consumer.ActiveData.Member));
				string stmt = "";
				if (_lagcolumn.Length >0)
				{
					if (_function==eFunction.Lag)
						stmt = string.Format("lag{1}_{0} = lag{1}({0});",_lagcolumn,_n.ToString());
					else
						stmt = string.Format("dif{1}_{0} = dif{1}({0});",_lagcolumn, _n.ToString());
				}
				
				codetemplate = codetemplate.Replace("{function_assign}",stmt);
				return codetemplate;
			}
		}

		public bool Initialize()
		{
			// initialize this instance of your task
			return true;
		}

		public SAS.Shared.AddIns.ShowResult Show(System.Windows.Forms.IWin32Window Owner)
		{
			// Show the default form for this custom task
			LagForm dlg = new LagForm();
			dlg.Consumer = this.consumer;
			dlg.Text = sLabel;
			
			if (_lagcolumn.Length>0)
				dlg.LagColumn = _lagcolumn;

			dlg.Function = _function;

			dlg.n = _n;

			if (dlg.ShowDialog()==System.Windows.Forms.DialogResult.OK)
			{
				_lagcolumn = dlg.LagColumn;
				_function = dlg.Function;
				_n = dlg.n;

				return SAS.Shared.AddIns.ShowResult.RunNow;
			}
			else
				return SAS.Shared.AddIns.ShowResult.Canceled;
		}

		// The label this task is known by within a project
		public string Label
		{
			get
			{			
				return sLabel;
			}
			set
			{
				sLabel = value;
			}
		}

		public int OutputDataCount
		{
			get
			{				
				// does this task create any output data sets?
				return 1;
			}
		}

		#endregion

		// Augment these methods to save and retrieve the state of your task
		#region Helper methods for serialization 

		private string WriteXML()
		{
			StringWriter sw = new StringWriter();

			XmlTextWriter writer = new XmlTextWriter(sw);
			writer.WriteStartElement("Lag");
			writer.WriteElementString("Function", XmlConvert.ToString((int)_function));
			writer.WriteElementString("Column", _lagcolumn);
			writer.WriteElementString("N", XmlConvert.ToString(_n));
			writer.WriteEndElement();
			writer.Close();

			return sw.ToString();
		}

		private void ReadXML(string xml)
		{
			if (xml!=null && xml.Length>0)
			{
				try
				{
					StringReader sr = new StringReader(xml);
					XmlTextReader reader = new XmlTextReader(sr);
					reader.ReadStartElement("Lag");
					_function = (eFunction) XmlConvert.ToInt32(reader.ReadElementString("Function"));
					_lagcolumn = reader.ReadElementString("Column");
					_n =  XmlConvert.ToInt32(reader.ReadElementString("N"));
					reader.ReadEndElement();
					reader.Close();
				}
				catch
				{
				}
			}
		}

		#endregion
	}
}
