%global dataset obs;

%macro charact(lib=WORK,dsn=_ALL_,catobs=30);

  data freq;
    stop;
  run;

  data univ;
    stop;
  run;

  proc contents data=&lib..&dsn. out=cont noprint;
  run;

  data _null_;
    set cont nobs=numobs;
    call symput('numobs',numobs);
  run;

  %do i=1 %to &numobs.;

    data _null_;
      pointer=&i.;
      set cont point=pointer;
      call symput('mem',memname);
      call symput('var',name);
      call symput('type',type);
      call symput('label',label);
      call symput('format',format);
      stop;
    run;

    %if &type.=1 %then %do;
      proc univariate data=&lib..&mem. noprint;
        var &var.;
        output out=tempuniv n=N nmiss=NMiss mean=Mean min=Min max=Max median=Median  
                            stdmean=StdMean sum=Total;
      run;

      data univ;
        set univ
            tempuniv(in=intemp)
            ;
        if intemp then do;
          Variable="&var.";
          label="&label.";
          dataset="&mem.";
          format="&FORMAT.";
        end;
      run;
    %end;

    %else %do;
      proc freq data=&lib..&mem. noprint;
        tables &var./missing out=tempfreq;
      run;

      data freq;
        length value $ 30;
        set freq
            tempfreq(in=intemp)
            ;
        if intemp then do;
          value=&var.;
          variable="&var.";
          label="&label.";
          dataset="&mem.";
          format="&FORMAT.";
        end;
        drop &var.;
      run;
    %end;
  %end;

  proc sort data=freq;
    where dataset ne ' ';
    by dataset variable label descending count;
  run;

  data freq;
    retain i newcount newperc 0;
    set freq;
    by dataset variable;
	Column=Variable;
    if value=' ' then value='***Missing***';
    if first.variable then i=1;
    else i=i+1;
    if i>%eval(&catobs.) then do;
      newcount=newcount+count;
      newperc=newperc+percent;
    end;
    if i<=%eval(&catobs.) or last.variable;
    if last.variable & i>%eval(&catobs.) then do;
      value='***All other values!!!***';
      count=newcount;
      percent=newperc;
      newcount=0;
      newperc=0;
    end;
	drop variable;
  run;
  options missing=' ' pageno=1;

  proc print data=freq label;
    title 
"Summary of Categorical Variables for &LIB..&DSN.- &catobs. 
Most Frequent Unique Values";
    by dataset Column label;
    id dataset Column label;
    var value count percent;
    label value="Value"
          dataset="Dataset"
          variable="Variable"
          label="Label"
          ;
  run;

  proc sort data=univ;
    by dataset variable label;
  run;

  proc print data=univ;
    where format not in ("DOLLAR","YYQ","MMDDYY","DATE","DDMMYY","YYMMDD","DATETIME");
    title "Summary of Numeric Variables for &LIB..&DSN.";
    by dataset variable label;
    id dataset variable label;
    var n nmiss total min mean median max stdmean;
  run;

  proc print data=univ;
    where format in ("DOLLAR");
    title "Summary of US Dollar Variables for &LIB..&DSN.";
    by dataset variable label;
    id dataset variable label;
    var n nmiss total min mean median max stdmean;
    format total mean stdmean min median max DOLLAR16.2;
  run;

  proc print data=univ;
    where format in ("YYQ","MMDDYY","DATE","DDMMYY","YYMMDD","DATETIME");
    title "Summary of Date Variables for &LIB..&DSN.";
    by dataset variable label;
    id dataset variable label;
    var n nmiss min median max;
    format min median max date.;
  run;

  proc datasets library=work nolist;
    delete cont tempfreq tempuni;
  run;

%mend charact; 

