// ---------------------------------------------------------------
// Copyright 2004, SAS Institute Inc.
// ---------------------------------------------------------------
using System;
using System.Windows.Forms;
using System.Xml;		// for XMLTextWriter and XMLTextReader
using System.IO;		// for StreamReader and StreamWriter
using System.Text;		// for StringBuilder

using SAS.Shared.AddIns;	// for access to the Add-in interfaces

namespace EGAddin.ClassicHat
{
	/// <summary>
	/// Summary description for ClassicHat.
	/// </summary>
	public class ClassicHat : ISASTask, ISASTaskDescription, ISASTaskAddIn
	{
		public ClassicHat()
		{
		}

		private int hatsize = 5;

		#region ISASTaskAddIn Members

		public bool VisibleInManager
		{
			get
			{
				return true;
			}
		}

		public string AddInName
		{
			get
			{
				return "Classic Hat";
			}
		}

		public void Disconnect()
		{
		}

		public int Languages(out string[] Items)
		{
			Items = new string[] { "en-US" };
			return 1;
		}

		public string AddInDescription
		{
			get
			{
				return "A very simple add-in example.";
			}
		}

		public bool Connect(ISASTaskConsumer Consumer)
		{

			return true;
		}

		public string Language
		{
			set
			{
				// we support english only
			}
		}

		#endregion

		#region ISASTaskDescription Members

		public string ProductsRequired
		{
			get
			{
				return "BASE";
			}
		}

		public bool StandardCategory
		{
			get
			{
				return false;
			}
		}

		public bool GeneratesListOutput
		{
			get
			{
				return true;
			}
		}

		public string TaskName
		{
			get
			{
				return "Classic Hat";
			}
		}

		public string Validation
		{
			get
			{
				return "SAS Sample";
			}
		}

		public string TaskCategory
		{
			get
			{
				return Global.Category;
			}
		}

		public string IconAssembly
		{
			get
			{
				// return the full path/name of this assembly
				return System.Reflection.Assembly.GetExecutingAssembly().Location;
			}
		}

		public string Clsid
		{
			get
			{
				return "9f5ee3b1-1057-44c3-b57d-57c1c29f5a0a";
			}
		}

		public SAS.Shared.AddIns.ShowType TaskType
		{
			get
			{
				return SAS.Shared.AddIns.ShowType.Wizard;
			}
		}

		public bool RequiresData
		{
			get
			{
				return false;
			}
		}

		public string IconName
		{
			get
			{
				// return the name of the icon within our assembly
				// including namespace qualifiers
				return "EGAddin.ClassicHat.SurfaceGraph.ico";
			}
		}

		public int MinorVersion
		{
			get
			{
				return 0;
			}
		}

		public int MajorVersion
		{
			get
			{
				return 1;
			}
		}

		public int NumericColsRequired
		{
			get
			{
				return 0;
			}
		}

		public string TaskDescription
		{
			get
			{
				return "The classic hat example from SAS.";
			}
		}

		public bool GeneratesSasCode
		{
			get
			{
				return true;
			}
		}

		public string FriendlyName
		{
			get
			{
				return "Classic Hat";
			}
		}

		public string ProcsUsed
		{
			get
			{
				return "DATA step; G3D";
			}
		}

		public string WhatIsDescription
		{
			get
			{
				return "Add some stuff here";
			}
		}

		public string ProductsOptional
		{
			get
			{
				return "";
			}
		}

		#endregion

		#region ISASTask Members

		public string RunLog
		{
			get
			{
				return null;
			}
		}

		public string XmlState
		{
			get
			{
				return WriteXML();
			}
			set
			{
				ReadXML(value);
			}
		}

		public void Terminate()
		{
			// TODO:  cleanup for this run
		}

		public SAS.Shared.AddIns.OutputData OutputDataInfo(int Index, out string Source, out string Label)
		{
			Source = null;
			Label = null;
			return new SAS.Shared.AddIns.OutputData();
		}

		public string SasCode
		{
			get
			{
				StringBuilder sb = new StringBuilder();
				sb.Append("data hat;\n");
				sb.AppendFormat("\tdo x = -{0} to {0} by .5;\n", hatsize);
				sb.AppendFormat("\t\tdo y = -{0} to {0} by .5;\n", hatsize);
				sb.Append("\t\tz = sin(sqrt(y*y + x*x));\n");
				sb.Append("\t\toutput;\n");
				sb.Append("\tend;\n");
				sb.Append("end;\n");
				sb.Append("proc g3d; plot y*x=z; run;\n");

				return sb.ToString();
			}
		}

		public bool Initialize()
		{
			// TODO:  init
			return true;
		}

		public SAS.Shared.AddIns.ShowResult Show(System.Windows.Forms.IWin32Window Owner)
		{
			// launch UI

			// create the dialog class
			ClassicHatForm dlg = new ClassicHatForm();

			dlg.HatSize = hatsize;
			
			if (DialogResult.OK == dlg.ShowDialog())
			{
				hatsize = dlg.HatSize;
				return SAS.Shared.AddIns.ShowResult.RunNow;
			}
			else
				return SAS.Shared.AddIns.ShowResult.Canceled;
		}

		public string Label
		{
			get
			{
				return "Classic Hat";
			}
			set
			{
			}
		}

		public int OutputDataCount
		{
			get
			{		
				return 0;
			}
		}

		#endregion

		#region Save and load XML state
		private string WriteXML()
		{
			StringWriter sw = new StringWriter();

			XmlTextWriter writer = new XmlTextWriter(sw);
			writer.WriteStartElement("ClassicHat");
			writer.WriteElementString("Size", XmlConvert.ToString(hatsize));
			writer.WriteEndElement();
			writer.Close();

			return sw.ToString();
		}

		private void ReadXML(string xml)
		{
			if (xml!=null && xml.Length>0)
			{
				try
				{
					StringReader sr = new StringReader(xml);
					XmlTextReader reader = new XmlTextReader(sr);
					reader.ReadStartElement("ClassicHat");
					hatsize = XmlConvert.ToInt32(reader.ReadElementString("Size"));
					reader.ReadEndElement();
					reader.Close();
				}
				catch
				{
				}
			}
		}
		#endregion
	}
}
