// ---------------------------------------------------------------
// Copyright 2004, SAS Institute Inc.
// ---------------------------------------------------------------
using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

namespace EGAddin.Contents
{
	/// <summary>
	/// Summary description for ContentsForm.
	/// </summary>
	public class ContentsForm : System.Windows.Forms.Form
	{
		private System.Windows.Forms.Button btnCancel;
		private System.Windows.Forms.Button btnOK;
		private System.Windows.Forms.GroupBox gbOutput;
		private System.Windows.Forms.RadioButton btnReport;
		private System.Windows.Forms.RadioButton btnData;
		private System.Windows.Forms.RadioButton btnBoth;
		private System.Windows.Forms.CheckBox chkDetails;
		private System.Windows.Forms.CheckBox chkIgnoreCase;
		
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public ContentsForm()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			//
			// TODO: Add any constructor code after InitializeComponent call
			//
		}

		private bool showDetails = true;
		public bool ShowDetails
		{
			set
			{
				showDetails = value;
			}
			get
			{
				return showDetails;
			}
		}

		private bool ignoreCase = true;
		public bool IgnoreCase
		{
			set
			{
				ignoreCase = value;
			}
			get
			{
				return ignoreCase;
			}
		}

		private Contents.OutputFormat outputFormat;
		public Contents.OutputFormat OutputFormat
		{
			set
			{
				outputFormat = value;
			}
			get
			{
				return outputFormat;
			}
		}

		protected override void OnLoad(EventArgs e)
		{
			// initialize the controls with the current values
			chkDetails.Checked = showDetails;
			chkIgnoreCase.Checked = ignoreCase;
			switch (outputFormat)
			{
				case Contents.OutputFormat.Data: btnData.Checked = true; break;
				case Contents.OutputFormat.Report: btnReport.Checked = true; break;
				case Contents.OutputFormat.ReportAndData: btnBoth.Checked = true; break;
				default:
					break;
			}

			base.OnLoad (e);
		}

		protected override void OnClosing(CancelEventArgs e)
		{
			// update properties with button states
			showDetails = chkDetails.Checked;
			ignoreCase = chkIgnoreCase.Checked;
			if (btnData.Checked) outputFormat = Contents.OutputFormat.Data;
			else if (btnReport.Checked) outputFormat = Contents.OutputFormat.Report;
			else if (btnBoth.Checked) outputFormat = Contents.OutputFormat.ReportAndData;

			base.OnClosing (e);
		}


		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(ContentsForm));
			this.btnCancel = new System.Windows.Forms.Button();
			this.btnOK = new System.Windows.Forms.Button();
			this.gbOutput = new System.Windows.Forms.GroupBox();
			this.btnReport = new System.Windows.Forms.RadioButton();
			this.btnData = new System.Windows.Forms.RadioButton();
			this.btnBoth = new System.Windows.Forms.RadioButton();
			this.chkDetails = new System.Windows.Forms.CheckBox();
			this.chkIgnoreCase = new System.Windows.Forms.CheckBox();
			this.gbOutput.SuspendLayout();
			this.SuspendLayout();
			// 
			// btnCancel
			// 
			this.btnCancel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.btnCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.btnCancel.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnCancel.Location = new System.Drawing.Point(208, 213);
			this.btnCancel.Name = "btnCancel";
			this.btnCancel.TabIndex = 16;
			this.btnCancel.Text = "Cancel";
			// 
			// btnOK
			// 
			this.btnOK.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.btnOK.DialogResult = System.Windows.Forms.DialogResult.OK;
			this.btnOK.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnOK.Location = new System.Drawing.Point(123, 213);
			this.btnOK.Name = "btnOK";
			this.btnOK.TabIndex = 15;
			this.btnOK.Text = "OK";
			// 
			// gbOutput
			// 
			this.gbOutput.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.gbOutput.Controls.Add(this.btnBoth);
			this.gbOutput.Controls.Add(this.btnData);
			this.gbOutput.Controls.Add(this.btnReport);
			this.gbOutput.Location = new System.Drawing.Point(9, 85);
			this.gbOutput.Name = "gbOutput";
			this.gbOutput.Size = new System.Drawing.Size(272, 119);
			this.gbOutput.TabIndex = 17;
			this.gbOutput.TabStop = false;
			this.gbOutput.Text = "Select the output format";
			// 
			// btnReport
			// 
			this.btnReport.Location = new System.Drawing.Point(17, 22);
			this.btnReport.Name = "btnReport";
			this.btnReport.Size = new System.Drawing.Size(175, 21);
			this.btnReport.TabIndex = 0;
			this.btnReport.Text = "Report";
			// 
			// btnData
			// 
			this.btnData.Location = new System.Drawing.Point(17, 51);
			this.btnData.Name = "btnData";
			this.btnData.Size = new System.Drawing.Size(175, 21);
			this.btnData.TabIndex = 1;
			this.btnData.Text = "Data set";
			// 
			// btnBoth
			// 
			this.btnBoth.Location = new System.Drawing.Point(17, 80);
			this.btnBoth.Name = "btnBoth";
			this.btnBoth.Size = new System.Drawing.Size(175, 21);
			this.btnBoth.TabIndex = 2;
			this.btnBoth.Text = "Both report and data set";
			// 
			// chkDetails
			// 
			this.chkDetails.Location = new System.Drawing.Point(21, 18);
			this.chkDetails.Name = "chkDetails";
			this.chkDetails.Size = new System.Drawing.Size(161, 24);
			this.chkDetails.TabIndex = 18;
			this.chkDetails.Text = "Include all data details";
			// 
			// chkIgnoreCase
			// 
			this.chkIgnoreCase.Location = new System.Drawing.Point(21, 48);
			this.chkIgnoreCase.Name = "chkIgnoreCase";
			this.chkIgnoreCase.Size = new System.Drawing.Size(227, 24);
			this.chkIgnoreCase.TabIndex = 19;
			this.chkIgnoreCase.Text = "Ignore case when sorting variable list";
			// 
			// ContentsForm
			// 
			this.AcceptButton = this.btnOK;
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.CancelButton = this.btnCancel;
			this.ClientSize = new System.Drawing.Size(292, 247);
			this.Controls.Add(this.chkIgnoreCase);
			this.Controls.Add(this.chkDetails);
			this.Controls.Add(this.gbOutput);
			this.Controls.Add(this.btnCancel);
			this.Controls.Add(this.btnOK);
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "ContentsForm";
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent;
			this.Text = "Contents of Data";
			this.gbOutput.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion
	}
}
