// ---------------------------------------------------------------
// Copyright 2004, SAS Institute Inc.
// ---------------------------------------------------------------
using System;
using System.Windows.Forms;
using System.Xml;		// for XMLTextWriter and XMLTextReader
using System.IO;		// for StreamReader and StreamWriter
using System.Text;		// for StringBuilder
using SAS.Shared.AddIns;	// for access to the Add-in interfaces
using EGAddin.RemoteLE;
namespace EGAddin.LifeExpectancy
{
	/// <summary>
	/// Summary description for LifeExpectancy.
	/// </summary>
	public class LifeExpectancy : ISASTaskAddIn, ISASTaskDescription, ISASTask
	{

		public enum LEGender
		{
			Male,
			Female
		}

		#region private members 

		// consumer object supplied by host application
		private ISASTaskConsumer consumer = null;

		private int age = 30;
		private LEGender gender = LEGender.Male;

		private string datacards = "";
		private int agelasttime = 0;
		private LEGender genderlasttime = LEGender.Male;
		private bool bShowReport = false;

		#endregion
		
		public LifeExpectancy()
		{
		}

		#region ISASTaskAddIn Members

		public bool VisibleInManager
		{
			get
			{
				return true;
			}
		}

		public string AddInName
		{
			get
			{
				return "Life Expectancy Table";
			}
		}

		public void Disconnect()
		{
			consumer = null;
		}

		public int Languages(out string[] Items)
		{
			Items = new string[] { "en-US" };
			return 1;
		}

		public string AddInDescription
		{
			get
			{
				return "Uses a web service to retrieve life expectancy data";
			}
		}

		public bool Connect(ISASTaskConsumer Consumer)
		{
			consumer = Consumer;
			return true;
		}

		public string Language
		{
			set
			{
				// no implementation
			}
		}

		#endregion

		#region ISASTaskDescription Members

		public string ProductsRequired
		{
			get
			{
				return "BASE";
			}
		}

		public bool StandardCategory
		{
			get
			{
				return false;
			}
		}

		public bool GeneratesListOutput
		{
			get
			{
				return true;
			}
		}

		public string TaskName
		{
			get
			{
				return "Life Expectancy";
			}
		}

		public string Validation
		{
			get
			{
				return "SAS Sample";
			}
		}

		public string TaskCategory
		{
			get
			{
				return Global.Category;
			}
		}

		public string IconAssembly
		{
			get
			{
				// return the full path/name of this assembly
				return System.Reflection.Assembly.GetExecutingAssembly().Location;
			}
		}

		public string Clsid
		{
			get
			{
				return "ce2eb0e4-d300-4967-87c5-71e5adcc89ed";
			}
		}

		public SAS.Shared.AddIns.ShowType TaskType
		{
			get
			{
				return SAS.Shared.AddIns.ShowType.Wizard;
			}
		}

		public bool RequiresData
		{
			get
			{
				return false;
			}
		}

		public string IconName
		{
			get
			{
				// return the name of the icon within our assembly
				// including namespace qualifiers
				return "EGAddin.LifeExpectancy.Stop.ico";
			}
		}

		public int MinorVersion
		{
			get
			{
				return 0;
			}
		}

		public int MajorVersion
		{
			get
			{
				return 1;
			}
		}

		public int NumericColsRequired
		{
			get
			{
				return 0;
			}
		}

		public string TaskDescription
		{
			get
			{
				return "Uses a web service (at www.xignite.com) to retrieve life expectancy data for the given age and gender.";
			}
		}

		public bool GeneratesSasCode
		{
			get
			{
				return true;
			}
		}

		public string FriendlyName
		{
			get
			{
				return "Life Expectancy";
			}
		}

		public string ProcsUsed
		{
			get
			{
				return "DATA step";
			}
		}

		public string WhatIsDescription
		{
			get
			{
				return "Uses a web service (at www.xignite.com) to retrieve life expectancy data for the given age and gender.";
			}
		}

		public string ProductsOptional
		{
			get
			{
				return "";
			}
		}

		#endregion

		#region ISASTask Members

		public string RunLog
		{
			get
			{			
				return null;
			}
		}

		public string XmlState
		{
			get
			{
				return WriteXML();
			}
			set
			{
				ReadXML(value);
			}
		}

		public void Terminate()
		{
			// TODO:  Add LifeExpectancy.Terminate implementation
		}

		public SAS.Shared.AddIns.OutputData OutputDataInfo(int Index, out string Source, out string Label)
		{
			Source = string.Format("WORK.LE{0}{1}",gender.ToString()[0],age.ToString());
			Label = string.Format("Life expectancy for {0}, age {1}", gender.ToString(), age);
			return SAS.Shared.AddIns.OutputData.Libref;
		}

		public string SasCode
		{
			get
			{
				string cards = GetLifeExpectancyData();
				string data=string.Format("WORK.LE{0}{1}",gender.ToString()[0],age.ToString());
				string cmt = string.Format("/* Life Expectancy Table for {0} - Age {1} */\n", gender.ToString(), age);
				cmt += string.Format("/* Retrieved from webservice hosted at www.xignite.com on {0} */\n", DateTime.Now.ToShortDateString());
				string ds = string.Format("DATA {0}; \nlength Age 8 Alive 8 Dead 8 DieThatYear 8;\ninput Age Alive Dead DieThatYear; \ncards;\n", data);
				string report="";
				if (bShowReport)
				{
					report = "\nRUN; \nPROC PRINT NOOBS LABEL;\n";
				}
				return cmt + ds + cards + report + "\n;\nRUN;";
			}
		}

		public bool Initialize()
		{
			return true;
		}

		public SAS.Shared.AddIns.ShowResult Show(IWin32Window Owner)
		{
			// launch UI

			// create the dialog class
			LifeExpectancyForm dlg = new LifeExpectancyForm();

			dlg.Age = age;
			dlg.Gender = gender;
			dlg.ShowReport = bShowReport;

			if (DialogResult.OK == dlg.ShowDialog())
			{
				age = dlg.Age;
				gender = dlg.Gender;
				bShowReport = dlg.ShowReport;

				return SAS.Shared.AddIns.ShowResult.RunNow;
			}
			else
				return SAS.Shared.AddIns.ShowResult.Canceled;
		}

		public string Label
		{
			get
			{		
				return "Life expectancy";
			}
			set
			{
			}
		}

		public int OutputDataCount
		{
			get
			{
				return 1;
			}
		}

		#endregion

		#region Access web service

		private string GetLifeExpectancyData()
		{
			if (agelasttime == age && genderlasttime == gender && datacards.Length>0)
				return datacards;

			StringBuilder sb = new StringBuilder();

			try
			{
				XigniteRetirement gled = new XigniteRetirement();

				RemoteLE.LifeExpectancy le = gled.GetLifeExpectancy(age, (gender == LEGender.Male) ? Gender.Male : Gender.Female);

				if (le.Outcome == OutcomeTypes.Success)
				{
					foreach (RemoteLE.DeathProbability dp in le.ProbabilityToDie)
					{
						double diethatyear = (dp.DieThatYear>1) ? 1 : dp.DieThatYear; // coverup for a bug in the service!
						sb.AppendFormat("{0} {1} {2} {3} {4}", dp.Age, dp.Alive, dp.Dead, diethatyear, Environment.NewLine);
					}
				}
				else
				{
					string error;
					switch (le.Outcome)
					{
						case OutcomeTypes.RegistrationError: 
							error = "Error: Not registered to use this web service"; 
							break;
						case OutcomeTypes.RequestError: 
							error = "Error: Request error occurred at web service"; 
							break;
						case OutcomeTypes.SystemError: 
							error = "Error: System error occurred at web service"; 
							break;
						default:
							error = "Error: Web service request failed: reason unknown"; 
							break;
					}
					sb.AppendFormat("* {0};\n",error);

				}
			}
			catch (Exception ex)
			{
				sb.AppendFormat("* Error: Web service request failed: {0};\n", ex.Message); 
				sb.AppendFormat("/* {0} */\n", ex.ToString()); 
			}

			// store answer away for later
			datacards = sb.ToString();
			agelasttime = age;
			genderlasttime = gender;

			return sb.ToString();
		}
		#endregion

		#region Save and load XML state
		private string WriteXML()
		{
			StringWriter sw = new StringWriter();

			XmlTextWriter writer = new XmlTextWriter(sw);
			writer.WriteStartElement("LifeExpectancy");
			writer.WriteElementString("age", XmlConvert.ToString(age));
			writer.WriteElementString("gender", gender.ToString());
			string cd = datacards.ToString().Replace("\\r\\n", "_nl_");
			writer.WriteElementString("cacheddata", cd);
			writer.WriteElementString("report", XmlConvert.ToString(bShowReport));
			writer.WriteEndElement();
			writer.Close();

			return sw.ToString();
		}

		private void ReadXML(string xml)
		{
			if (xml!=null && xml.Length>0)
			{
				try
				{
					StringReader sr = new StringReader(xml);
					XmlTextReader reader = new XmlTextReader(sr);
					reader.ReadStartElement("LifeExpectancy");
					age = XmlConvert.ToInt32(reader.ReadElementString("age"));
					string g = reader.ReadElementString("gender");
					gender = (g=="Male") ? LEGender.Male : LEGender.Female;
					datacards = reader.ReadElementString("cacheddata");
					datacards = datacards.Replace("_nl_", "\\r\\n");
					bShowReport = XmlConvert.ToBoolean(reader.ReadElementString("report"));
					reader.ReadEndElement();
					reader.Close();
				}
				catch
				{
				}
			}
		}
		#endregion

	}
}
